/********************************************************************************

	main.c

	display stand controled by USB radio mouse.

    setting I/O pin
		MCLR    (1) MCLR    AVDD (28) +3V3, via 0.1uF to GND
        U1TX    (2) RP5     AVSS (27) GND
        U1RX    (3) RP6     RP15 (26) OC1 Turn Motor Control
        PGED1   (4) RB0     RB14 (25) Turn Motor IN1
        PGEC1   (5) RB1     RB13 (24) Turn Motor IN2 
                (6) RB2     VUSB (23) +3V3
                (7) RB3     D-   (22) USB D-
        GND     (8) VSS     D+   (21) USB D+
detect  rotate  (9) RA2     Vcap (20) via 10uF to GND
               (10) RA3  DISVREG (19) GND
               (11) RB4     RB9  (18)  
               (12) RA4     RP8  (17) OC3 Lift Up Servo Motor signal
        +3V3   (13) VDD     RP7  (16) OC2 Voltage Control for External 5V OUT 
               (14) RB5     VBUS (15) USB VBUS


********************************************************************************/

#include <p24FJ64GB002.h>
#include "GenericTypeDefs.h"
#include "pic_232.h"
#include "pic_print.h"
#include "pic_string.h"
#include "pic_delay.h"
#include "usb_mouse.h"

// *** CHIP CONFIGURATION *******************************************************
_CONFIG1(WDTPS_PS1 & FWPSA_PR32 & WINDIS_OFF & FWDTEN_OFF & ICS_PGx1 & GWRP_OFF & GCP_OFF & JTAGEN_OFF)
_CONFIG2(POSCMOD_NONE & I2C1SEL_PRI & IOL1WAY_OFF & OSCIOFNC_ON & FCKSM_CSDCMD & FNOSC_FRCPLL & PLL96MHZ_ON & PLLDIV_DIV2 & IESO_OFF)
_CONFIG3(WPFP_WPFP0 & SOSCSEL_IO & WUTSEL_LEG & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM)
//_CONFIG3(WPFP_WPFP0 & SOSCSEL_SOSC & WUTSEL_LEG & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM)
_CONFIG4(DSWDTPS_DSWDTPS3 & DSWDTOSC_LPRC & RTCOSC_SOSC & DSBOREN_OFF & DSWDTEN_OFF)

// *** macro ***********************************************************************

#define	DEBUG_MODE
#ifndef DEBUG_MODE
	#define	rs_putstr()
	#define myprintf()
#endif

#define	_EI()	asm("DISI #0")
#define	_DI()	asm("DISI #200")

#define	turn_stop()		{ LATBbits.LATB13 = 0; LATBbits.LATB14 = 0; delay_5us(); }
#define	turn_right(x)	{ LATBbits.LATB13 = 1; LATBbits.LATB14 = 0; OC1R = x; }
#define	turn_left(x)	{ LATBbits.LATB13 = 0; LATBbits.LATB14 = 1; OC1R = x; }
#define	lift_position	OC3R
#define	power_position	OC2R
#define	TURN_DETECT		PORTAbits.RA2

#define	MANUAL_STAND_OPERATION	0
#define	MANUAL_POWER_OPERATION	1
#define	AUTO_STAND_OPERATION	2
#define	AUTO_POWER_OPERATION	3

#define	TURN_MINIMUM_SPEED	100
#define	TURN_MAXIMUM_SPEED	1000
#define	TURN_SPEED_STEPS	50

#define	TURN_TIMER	0
#define LIFT_TIMER	1
#define	POWER_TIMER	2
#define	TIMERS		3

#define DIRECT_RIGHT 	0	// mouse right rotate
#define	DIRECT_LEFT		1	// mouse left rotate

typedef enum {
	POWER_OFF_STATE = 0,
	POWER_OFF_SUB_STATE,
	POWER_ON_STATE,
	POWER_ON_SUB_STATE,
	POWER_UP_STATE,
	POWER_DOWN_STATE,
} POWER_STATE;

typedef enum	{
	TURN_STOP_STATE = 0,
	TURN_RIGHT_STATE,
	TURN_RIGHT_SUB_STATE,
	TURN_RIGHT_MAX_STATE,
	TURN_LEFT_STATE,
	TURN_LEFT_SUB_STATE,
	TURN_LEFT_MAX_STATE,
} TURN_STATE;

typedef enum {
	LIFT_UP_STATE = 0,
	LIFT_UP_SUB_STATE,
	LIFT_UP_MAX_STATE,
	LIFT_AUTO_UP_STATE,
	LIFT_AUTO_UP_SUB_STATE,
	LIFT_DOWN_STATE,
	LIFT_DOWN_SUB_STATE,
	LIFT_DOWN_MAX_STATE,
	LIFT_AUTO_DOWN_STATE,
	LIFT_AUTO_DOWN_SUB_STATE,
} LIFT_STATE;



// *** Function Prototype *******************************************************
void	system_initialize(void);
BOOL 	USBHostInit(unsigned long flags);
void	set_external_power(int t_on, int t_off, int t_pwr, int t_rpt);
void	control_external_power(void);
void	detect_mouse_revolution(void);
void	turn_control(void);
void	lift_control(int ret);
void	power_control(void);


// *** Global Variable **********************************************************
USB_MOUSE_DATA	moused;
TURN_STATE	turn_state;
LIFT_STATE	lift_state;
POWER_STATE	power_state;

/*===============================================================================
	Timer T1 interrupt
	interval cycle 20msec
===============================================================================*/

static volatile unsigned int time_counter[TIMERS];
static volatile unsigned int turn_times, turn_detect_prev, turn_detect_val;
static volatile unsigned int is_control_external_power;

void __attribute__ ((__interrupt__, no_auto_psv)) _T1Interrupt(void)
{
	int	n;

	IFS0bits.T1IF = 0;	// clear interrupt flag

	// turn table turn time count	
	for(n = 0; n < TIMERS; n++){
		time_counter[n]++;
	}

	// external power control
	if(is_control_external_power){
		control_external_power();
	}


	// detect turn table rotate, count up times.
	//    down edge detect 

	// same during 20msec?
	if(TURN_DETECT == turn_detect_prev){
		// changed?
		if(TURN_DETECT != turn_detect_val){
			// save now val
			turn_detect_val = TURN_DETECT;
			// changed to low
			if(turn_detect_val == 0){
				turn_times++;
			}
		}
	}else{
		turn_detect_prev = TURN_DETECT;
	}

}


/*-------------------------------------------------------------------------------
	get_time_count()
-------------------------------------------------------------------------------*/

unsigned int get_time_count(int tmr)
{
	unsigned int val;

	_DI();
	val = time_counter[tmr];
	_EI();
	return val;
}


/*-------------------------------------------------------------------------------
	clear_time_count()
-------------------------------------------------------------------------------*/

void	clear_time_count(int tmr)
{
	_DI();
	time_counter[tmr] = 0;
	_EI();
}


/*-------------------------------------------------------------------------------
	get_turn_times()
-------------------------------------------------------------------------------*/

unsigned int get_turn_times(void)
{
	unsigned int val;

	_DI();
	val = turn_times;
	_EI();
	return val;
}


/*======================================================================
	external output Power voltage control

	t_on:	power on time (sec)	
	t_off:	power off time(sec)
	t_pwr:	power level (0-100)	0=off, 100=full, 50=half
	t_rpt:	repeat n times, n=0 forever

    start from power on state
======================================================================*/

static volatile unsigned long t_on_time, t_off_time, power_on_time, power_off_time;
static int power_level, power_repeat, t_repeat;

void	set_external_power(int t_on, int t_off, int t_pwr, int t_rpt)
{
	// stop the power operation
	_DI();
	is_control_external_power = 0;
	_EI();

	// set power control parameter
	power_on_time = t_on * 50UL;
	power_off_time = t_off * 50UL;
	power_level = t_pwr * 10;	// 100%=1000
	power_repeat = t_rpt;

	// forever Power Off mode?
	if(power_on_time == 0) return;

	// forever Power On mode?
	if(power_off_time == 0 && power_repeat == 0){
		power_position = power_level;
		return;
	}

	// go new power operation
	power_state = POWER_ON_STATE;
	t_on_time = power_on_time;
	t_repeat = power_repeat;
	power_position = power_level;

	_DI();
	is_control_external_power = 1;
	_EI();

}

//
// this function is called by interrupt routines 20msec interval.
//   one cycle is from POWER_ON to POWER_OFF
void	control_external_power(void)
{
	switch(power_state){
	case POWER_OFF_STATE:
		if(--t_off_time == 0){
			// check repeat
			if(t_repeat == 0 || --t_repeat){
				// repeat next
				power_state = POWER_ON_STATE;
				t_on_time = power_on_time;
				t_repeat = power_repeat;
				power_position = power_level;
			}else{
				// stop power operation
				is_control_external_power = 0;
			}
		}
		break;

	case POWER_ON_STATE:
		if(--t_on_time == 0){
			power_state = POWER_OFF_STATE;
			t_off_time = power_off_time;
			power_position = 0;	// power off
		}
		break;

	default:
		break;

	}
}



/*===============================================================================
	main
	This program start from here.
===============================================================================*/

int main (void)
{
	int	rc, operation, ret;

	// initialize system
	system_initialize();	// turn stop

	// opening message
	rs_putstr("\r\n*** Motor Drive test.\r\n"); 

	// set external power 
	// on=1sec off=0sec level=100% repeat=0(non stop)
	//set_external_power(1, 0, 100, 0);

	// Turn Table set Home Position
	rs_putstr("Turn Table set to Home Position.\r\n");
	rc = get_turn_times();
	myprintf("turn times=%d\r\n", rc);
	turn_right(TURN_MAXIMUM_SPEED/3);
	while(rc == get_turn_times()){
		delay_ms(1);
	}
	turn_stop();
	rs_putstr("complete set Home.\r\n");
	rc = get_turn_times();
	myprintf("turn times=%d\r\n", rc);

	// Lift Max Down
	lift_position = 2840;

	// Power full voltage out
	power_position = 1000;

	// initialize state
	turn_state = TURN_STOP_STATE;
	lift_state = LIFT_DOWN_MAX_STATE;
	power_state = POWER_ON_STATE;

	rs_putstr("Turn Stop.\r\n");
	rs_putstr("Lift Down Max.\r\n");
	rs_putstr("Power On.\r\n");

	operation = MANUAL_STAND_OPERATION;
	rs_putstr("Stand Operation.\r\n");

	// forever loop
	for(;;){

		// Read the Mouse Button and Axis 
		ret = mouse_read(&moused);

		// operation change ?
		// left, right both button push
		if(moused.button.left && moused.button.right){
			
			// wait for button release
			do{
				mouse_read(&moused);
			}while(moused.button.left || moused.button.right);

			if(operation == MANUAL_STAND_OPERATION){
				operation = MANUAL_POWER_OPERATION;
				rs_putstr("Power Operation.\r\n");
			}else if(operation == MANUAL_POWER_OPERATION){
				operation = MANUAL_STAND_OPERATION;
				rs_putstr("Stand Operation.\r\n");
			}
		}	

		if(operation == MANUAL_STAND_OPERATION){
			detect_mouse_revolution();
			turn_control();
			lift_control(ret);

		}else if(operation == MANUAL_POWER_OPERATION){
			power_control();

		}	

		#ifdef DEBUG_MODE
		if(rc != get_turn_times()){
			rc = get_turn_times();
			myprintf("turn times=%d\r\n", rc);
		}
		#endif
	}

}


/*-------------------------------------------------------------------------------
	detect mouse revolution

           to right                    to left 
        (1)   |   (2)               (3)   |  (4)
       x > 0  |  x > 0             x < 0  |  x < 0
       y < 0  |  y > 0             y > 0  |  y < 0
     ---------+----------        ---------+---------
       x < 0  |  x < 0             x > 0  |  x > 0
       y < 0  |  y > 0             y > 0  |  y < 0
        (4)   |   (3)               (2)   |  (1)

(1)->(2)->(3)->(4)->(1)->(2)    (4)->(3)->(2)->(1)->(4)->(3)

-------------------------------------------------------------------------------*/

static	unsigned char zone_change_direction;	// DIRECT_RIGHT or DIRECT_LEFT
static	unsigned char prev_zone;
static	unsigned char zone_change_count;
static	int		turn_speed;

void detect_mouse_revolution(void)
{
	unsigned char zone;

	// check in zone
	if(moused.x_axis < 0 && moused.y_axis < 0){
		zone = 4;
	}else if(moused.x_axis < 0 && moused.y_axis > 0){
		zone = 3;
	}else if(moused.x_axis > 0 && moused.y_axis < 0){
		zone = 1;
	}else if(moused.x_axis > 0 && moused.y_axis > 0){
		zone = 2;
	}else{
		return;
	}

	// zone didn't change?
	if(prev_zone == zone) return;

	// ***** changed zone

	// right check
	if((zone == 1 && prev_zone == 4) || (zone != 1 && (zone -1) == prev_zone)){
		// right!
		if(zone_change_direction == DIRECT_RIGHT){
			zone_change_count++;
			if(zone_change_count >= 4){
				if(turn_state == TURN_STOP_STATE){
					turn_speed = TURN_MINIMUM_SPEED;
					turn_right(turn_speed);
					turn_state = TURN_RIGHT_SUB_STATE;
					clear_time_count(TURN_TIMER);
					rs_putstr("Turn Right Sub.\r\n");
				}else if(turn_state == TURN_LEFT_SUB_STATE || 
						 turn_state == TURN_LEFT_MAX_STATE){
					turn_state = TURN_STOP_STATE;
					rs_putstr("Turn Stop.\r\n");
				}
				zone_change_count = 0;
			}
		}else{
			zone_change_direction = DIRECT_RIGHT;
			zone_change_count = 0;
		}

	// left check
	}else if((zone == 4 && prev_zone == 1) || (zone != 4 && (zone +1) == prev_zone)){
		// left!
		if(zone_change_direction == DIRECT_LEFT){
			zone_change_count++;
			if(zone_change_count >= 4){
				if(turn_state == TURN_STOP_STATE){
					turn_speed = TURN_MINIMUM_SPEED;
					turn_left(turn_speed);
					turn_state = TURN_LEFT_SUB_STATE;
					clear_time_count(TURN_TIMER);
					rs_putstr("Turn Left Sub.\r\n");
				}else if(turn_state == TURN_RIGHT_SUB_STATE ||
						turn_state == TURN_RIGHT_MAX_STATE){
					turn_state = TURN_STOP_STATE;
					rs_putstr("Turn Stop.\r\n");
				}
				zone_change_count = 0;
			}
		}else{
			zone_change_direction = DIRECT_LEFT;
			zone_change_count = 0;
		}
	}else{
		zone_change_count = 0;
	}
	// saved now zone
	prev_zone = zone;

}


/*-------------------------------------------------------------------------------
	void	turn_control(void)
-------------------------------------------------------------------------------*/

void	turn_control(void)
{
	int	tc;

	switch(turn_state){
	case TURN_STOP_STATE:
		// right and left both button push
		// continues stop state
		if(moused.button.right && moused.button.left){
			break;
		}
		// push right button only
		// change to right state
		if(moused.button.right){
			turn_speed = TURN_MINIMUM_SPEED;
			turn_right(turn_speed);
			turn_state = TURN_RIGHT_STATE;
			clear_time_count(TURN_TIMER);
			rs_putstr("Turn Right.\r\n");
			break;
		}
		// push left button only
		// change to left state
		if(moused.button.left){
			turn_speed = TURN_MINIMUM_SPEED;
			turn_left(turn_speed);
			turn_state = TURN_LEFT_STATE;
			clear_time_count(TURN_TIMER);
			rs_putstr("Turn Left.\r\n");
			break;
		}
		// release both button, continues stop state
		break;

	case TURN_RIGHT_STATE:
		// right button release or left button push
		// to stop state
		if(!moused.button.right){
			turn_stop();
			turn_state = TURN_STOP_STATE;
			rs_putstr("Turn Stop.\r\n");
			break;
		}
		// not break, threw TURN_RIGHT_SUB_STATE
	case TURN_RIGHT_SUB_STATE:

		if(moused.button.left){
			turn_stop();
			turn_state = TURN_STOP_STATE;
			rs_putstr("Turn Stop.\r\n");
			break;
		}

		// right button push to speed up
		tc = get_time_count(TURN_TIMER);
		//myprintf("tc=%d\r\n", tc);
		if(tc >= 12){	// 12*20msec=240msec
			turn_speed  += TURN_SPEED_STEPS;
			//myprintf("turn_speed=%d\r\n", turn_speed);
			if(turn_speed > TURN_MAXIMUM_SPEED){
				turn_speed = TURN_MAXIMUM_SPEED;
				turn_state = TURN_RIGHT_MAX_STATE;
				rs_putstr("Turn Right Max Speed.\r\n");
			}
			turn_right(turn_speed);
			clear_time_count(TURN_TIMER);
		}
		break;

	case TURN_RIGHT_MAX_STATE:
		// left button push to stop state
		if(moused.button.left){
			turn_stop();
			turn_state = TURN_STOP_STATE;
			rs_putstr("Turn Stop.\r\n");
		}
		break;

	case TURN_LEFT_STATE:
		// left button release or right button push
		// to stop state
		if(!moused.button.left){
			turn_stop();
			turn_state = TURN_STOP_STATE;
			rs_putstr("Turn Stop.\r\n");
			break;
		}
		// not break, threw TURN_LEFT_SUB_STATE
	case TURN_LEFT_SUB_STATE:

		if(moused.button.right){
			turn_stop();
			turn_state = TURN_STOP_STATE;
			rs_putstr("Turn Stop.\r\n");
			break;
		}

		// left button push to speed up
		tc = get_time_count(TURN_TIMER);
		if(tc >= 12){	// 12*20msec=240msec
			turn_speed  += TURN_SPEED_STEPS;
			if(turn_speed > TURN_MAXIMUM_SPEED){
				turn_speed = TURN_MAXIMUM_SPEED;
				turn_state = TURN_LEFT_MAX_STATE;
				rs_putstr("Turn Left Max Speed.\r\n");
			}
			turn_left(turn_speed);
			clear_time_count(TURN_TIMER);
		}
		break;

	case TURN_LEFT_MAX_STATE:
		// right button push to stop state
		if(moused.button.right){
			turn_stop();
			turn_state = TURN_STOP_STATE;
			rs_putstr("Turn Stop.\r\n");
		}
		break;

	default:
		break;
	}

}


/*-------------------------------------------------------------------------------
	void	lift_control(int ret)
-------------------------------------------------------------------------------*/

void	lift_control(int ret)
{
	int	val;
	unsigned int tc;

	switch(lift_state){

	case LIFT_DOWN_MAX_STATE:
		if(moused.wheel < 0){
			lift_state = LIFT_UP_STATE;
			rs_putstr("Lift Up.\r\n");
			val = moused.wheel * 5;
			val += lift_position;
			if(val > 2840){
				val = 2840;
				lift_state = LIFT_UP_MAX_STATE;
				rs_putstr("Lift Up Max.\r\n");
			}
			lift_position = val;
		}
		break;

	case LIFT_UP_MAX_STATE:
		if(moused.wheel > 0){
			lift_state = LIFT_DOWN_STATE;
			rs_putstr("Lift Down.\r\n");
			val = moused.wheel * 5;
			val += lift_position;
			if(val < 1800){
				val = 1800;
				lift_state = LIFT_DOWN_MAX_STATE;
				rs_putstr("Lift Down Max.\r\n");
			}
			lift_position = val;
		}
		break;


	case LIFT_UP_STATE:
	case LIFT_DOWN_STATE:

		if(moused.button.wheel){
			if(lift_state == LIFT_UP_STATE){
				lift_state = LIFT_UP_SUB_STATE;
			}else{
				lift_state = LIFT_DOWN_SUB_STATE;
			}
			break;
		}

		if(!ret || moused.wheel == 0) break;

		val = moused.wheel * 5;
		val += lift_position;
		if(moused.wheel < 0){
			lift_state = LIFT_UP_STATE;
		}else{
			lift_state = LIFT_DOWN_STATE;
		}
		if(val > 2840){
			val = 2840;
			lift_state = LIFT_DOWN_MAX_STATE;
			rs_putstr("Lift Down Max.\r\n");
		}else if(val < 1800){
			val = 1800;
			lift_state = LIFT_UP_MAX_STATE;
			rs_putstr("Lift Up Max.\r\n");
		}
		lift_position = val;
		break;


	case LIFT_UP_SUB_STATE:
		if(!moused.button.wheel){
			lift_state = LIFT_AUTO_UP_STATE;
			rs_putstr("Lift Auto Up.\r\n");
		}
		break;

	case LIFT_DOWN_SUB_STATE:
		if(!moused.button.wheel){
			lift_state = LIFT_AUTO_DOWN_STATE;
			rs_putstr("Lift Auto Down.\r\n");
		}
		break;


	case LIFT_AUTO_UP_STATE:

		if(moused.button.wheel){
			lift_state = LIFT_AUTO_UP_SUB_STATE;
			break;
		}

		tc = get_time_count(LIFT_TIMER);	
		if(tc >= 3){	// 3*20msec=60msec
			val = lift_position -10;
			if(val < 1800){
				val = 1800;
				lift_state = LIFT_UP_MAX_STATE;
				rs_putstr("Lift Up Max.\r\n");
			}
			lift_position = val;
			clear_time_count(LIFT_TIMER);
		}
		break;


	case LIFT_AUTO_DOWN_STATE:

		if(moused.button.wheel){
			lift_state = LIFT_AUTO_DOWN_SUB_STATE;
			break;
		}

		tc = get_time_count(LIFT_TIMER);	
		if(tc >= 3){	// 3*20msec=60msec
			val = lift_position +10;
			if(val > 2840){
				val = 2840;
				lift_state = LIFT_DOWN_MAX_STATE;
				rs_putstr("Lift Down Max.\r\n");
			}
			lift_position = val;
			clear_time_count(LIFT_TIMER);
		}
		break;


	case LIFT_AUTO_UP_SUB_STATE:
		if(!moused.button.wheel){
			lift_state = LIFT_UP_STATE;
			rs_putstr("Lift Up.\r\n");
		}
		break;

	case LIFT_AUTO_DOWN_SUB_STATE:
		if(!moused.button.wheel){
			lift_state = LIFT_DOWN_STATE;
			rs_putstr("Lift Down.\r\n");
		}
		break;

	default:
		break;
	}
}


/*-------------------------------------------------------------------------------
	external power manual control
-------------------------------------------------------------------------------*/

static	int	pwrlvl;

void	power_control(void)
{
	int	tc, val;

	switch(power_state){
	case POWER_OFF_SUB_STATE:
		if(!moused.button.wheel){
			power_state = POWER_OFF_STATE;
			pwrlvl = power_position;
			power_position = 0;
			rs_putstr("Power Off.\r\n");
		}
		break;

	case POWER_OFF_STATE:
		if(moused.button.wheel){
			power_state = POWER_ON_SUB_STATE;
		}
		break;

	case POWER_ON_SUB_STATE:
		if(!moused.button.wheel){
			power_state = POWER_ON_STATE;
			power_position = pwrlvl;
			rs_putstr("Power On.\r\n");
		}
		break;

	case POWER_ON_STATE:
		if(moused.button.wheel){
			power_state = POWER_OFF_SUB_STATE;
			break;
		}

		if(!moused.button.right && !moused.button.left){
			break;
		}	

		if(moused.button.right){
			power_state = POWER_UP_STATE;
			rs_putstr("Power Up.\r\n");
		}else{ /* if(moused.button.left){ */
			power_state = POWER_DOWN_STATE;
			rs_putstr("Power Down.\r\n");
		}
		clear_time_count(POWER_TIMER);
		break;


	case POWER_UP_STATE:
		if(!moused.button.right || moused.button.left){
			power_state = POWER_ON_STATE;
			rs_putstr("Stop Power Up.\r\n");
			break;
		}

		tc = get_time_count(POWER_TIMER);		
		if(tc >= 5){	// 5*20msec=100msec
			val = power_position +20; 
			if(val >= 1000) val = 1000;
			power_position = val;
			clear_time_count(POWER_TIMER);
		}
		break;

	case POWER_DOWN_STATE:
		if(!moused.button.left || moused.button.right){
			power_state = POWER_ON_STATE;
			rs_putstr("Stop Power Down.\r\n");
			break;
		}

		tc = get_time_count(POWER_TIMER);		
		if(tc >= 5){	// 5*20msec=100msec
			val = power_position -20; 
			if(val <= 0) val = 0;
			power_position = val;
			clear_time_count(POWER_TIMER);
		}
		break;

	default:
		break;
	}
}



/*======================================================================
	System Initialize ( setup peripheral module ) 


    setting I/O pin
		MCLR    (1) MCLR    AVDD (28) +3V3, via 0.1uF to GND
        U1TX    (2) RP5     AVSS (27) GND
        U1RX    (3) RP6     RP15 (26) OC1 Turn Motor Control
        PGED1   (4) RB0     RB14 (25) Turn Motor IN1
        PGEC1   (5) RB1     RB13 (24) Turn Motor IN2 
                (6) RB2     VUSB (23) +3V3
                (7) RB3     D-   (22) USB D-
        GND     (8) VSS     D+   (21) USB D+
detect  rotate  (9) RA2     Vcap (20) via 10uF to GND
               (10) RA3  DISVREG (19) GND
               (11) RB4     RB9  (18)  
               (12) RA4     RP8  (17) OC3 Lift Up Servo Motor signal
        +3V3   (13) VDD     RP7  (16) OC2 Voltage Control for External 5V OUT 
               (14) RB5     VBUS (15) USB VBUS


======================================================================*/

void	system_initialize(void)
{
	unsigned int pll_startup_counter;

	// Fosc(system clock) 8MHz postscale = 1:1 x4 = 32MHz, Fcy = Fosc/2
	CLKDIV = 0;

	// PLL clock start up
	pll_startup_counter = 600;
	CLKDIVbits.PLLEN = 1;
	while(pll_startup_counter--);

	//--- setting interrupt ---------------------------------------
	INTCON1 = 0;			// NSTDIS=0 nesting ok

	//--- setting I/O port ----------------------------------------
	AD1PCFG = 0xffff;	// Analog disable
	LATA = 0x0000; TRISA = 0xffff;	// RA are default input.
	LATB = 0x0000; TRISB = 0xffff;	// RB are default input.

	//--- remapping I/O pin for peripheral ------------------------
	//unlock registers
	__builtin_write_OSCCONL(OSCCON & 0xBF);

	// Assign UART1	
	RPOR2bits.RP5R = 3;		// RP5/RA0(pin 2)->UART1 TX
	RPINR18bits.U1RXR = 6;	// RP6/RA1(pin 3)->UART1 RX

	LATAbits.LATA0 = 1;		// RA0(U1TX) to level high
	TRISAbits.TRISA0 = 0;	// RA0(U1TX) is output
	TRISAbits.TRISA1 = 1;	// RA1(U1RX) is input

	// Assign Turn Table rotate detect
	TRISAbits.TRISA2 = 1;	// RA2 is input

	// Assign OC1 To Pin RP15(RB15) Turn Speed Control
	// Assign OC2 To Pin RP7(RB7) Voltage Control
	// Assign OC3 To Pin RP8(RB8) Servo Control
	RPOR7bits.RP15R = 18;	// OC1(18)
	RPOR3bits.RP7R = 19;	// OC2(19)
	RPOR4bits.RP8R = 20;	// OC3(20)

	TRISBbits.TRISB15 = 0;	// RB15 is outpput
	TRISBbits.TRISB7 = 0;	// RB7 is output
	TRISBbits.TRISB8 = 0;	// RB8 is output
	ODCB=0b0000000010000000; // RB7 is open drain. C1815 base input.

	// Assign Turn Motor Control IN1,IN2
	LATBbits.LATB13 = 0;	// Motor IN1 to level low
	LATBbits.LATB14 = 0;	// Motor IN2 to level low
	TRISBbits.TRISB13 = 0;	// RB13 is outpput
	TRISBbits.TRISB14 = 0;	// RB14 is output

	//lock registers
	__builtin_write_OSCCONL(OSCCON | 0x40);

	// --- set OC1 for turn motor driver control
	OC1CON1 = 0;				// OCM = 0, module disable
	OC1CON1bits.OCTSEL = 0b011;	// selected TMR5 timer clock
	OC1CON2 = 0;				// OCTRIG = 0, Synchronize OC2 with source designated by SYNCEL bits.
	OC1CON2bits.SYNCSEL = 0b11111;	// SYNCSEL = 31, Syncronization source is this module.
	OC1RS = 999;				// 2,000,000Hz/2,000Hz(0.5msec)=1,000
	OC1R = 0;					// OC2R 0 - 1,000, initial 0 stop
	OC1CON1bits.OCM = 0b110;

	// --- set OC2 for external out Voltage Control
	OC2CON1 = 0;				// OCM = 0, module disable
	OC2CON1bits.OCTSEL = 0b011;	// selected TMR5 timer clock
	OC2CON2 = 0;				// OCTRIG = 0, Synchronize OC2 with source designated by SYNCEL bits.
	OC2CON2bits.SYNCSEL = 0b11111;	// SYNCSEL = 31, Syncronization source is this module.
	OC2RS = 999;				// 2,000,000Hz/2,000Hz(0.5msec)=1,000
	OC2R = 1000;				// OC2R 0 - 1,000, initial 1000 full voltage out
	OC2CON1bits.OCM = 0b110;	

	// --- set OC3 for Lift Up Servo Motor Control
	OC3CON1 = 0;				// OCM = 0, module disable
	OC3CON1bits.OCTSEL = 0b011;	// selected TMR5 timer clock
	OC3CON2 = 0;				// OCTRIG = 0, Synchronize OC2 with source designated by SYNCEL bits.
	OC3CON2bits.SYNCSEL = 0b11111;	// SYNCSEL = 31, Syncronization source is this module.
	OC3RS = 39999;				// 2,000,000Hz/50Hz(20msec)=40,000
	//OC3R = 2999;				// 2,000,000Hz/666.66Hz(1.5msec)=3000
	//OC3R = 2599;				// 2,000,000Hz/769Hz(1.3msec)=2600
	OC3R = 2840;
	OC3CON1bits.OCM = 0b110;

	// --- set Timer5 for OC1,OC2,OC3,OC4 clock source 
	T5CON = 0;		// stop timer, prescale 1:1
	T5CONbits.TCKPS = 0b01;	// clock count prescale 1:8
	TMR5 = 0;		// TMR5 clock is 16MHz/8=2MHz
	PR5 = 0;
	T5CONbits.TON = 1; // TMR5 on

	//--- set Timer1 for turn, lift speed up timing control 
	T1CON = 0;
	T1CONbits.TCKPS = 0b10;	// clock count prescale 1:64
	TMR1 = 0;		// TMR1 clock is 16MHz/64=250KHz
	PR1 = 4999;		// 250KHz/50Hz=5000 20msec
	IEC0bits.T1IE = 1;	// enable T1 timer interrupt
	IFS0bits.T1IF = 0;	// clear interrupt flag
	T1CONbits.TON = 1;	// TMR1 on

	//--- setting UART -------------------------------------------
	// see pic_232.c
	rs_initialize();

	//--- setting USB --------------------------------------------
	// USB Host driver
	USBHostInit(0);

}

#if 0
		if(mouse_read(&moused)){
			myprintf("Left  Bt : %d \r\n", moused.button.left);
			myprintf("Right Bt : %d \r\n", moused.button.right);
			myprintf("CenterBt : %d \r\n", moused.button.center);
    		myprintf("X-Axis   : %d \r\n", moused.x_axis);
			myprintf("Y-Axis   : %d \r\n", moused.y_axis);
			myprintf("Wheel    : %d \r\n\r\n", moused.wheel);

			if(moused.wheel != 0){
				OC3R = OC3R + moused.wheel;
				if(OC3R > 2840) OC3R = 2840;
				if(OC3R < 1800) OC3R = 1800;
			}
			myprintf("OC3R=%d\r\n", OC3R);	
		}
#endif


/*** end of main.c ********************************************************************************/
